@@ -1,5 +1,11 @@
+# :stopdoc:
+
+class Hoe
+end
+
 module Hoe::Minitest
   def initialize_minitest
+    gem "minitest"
     require 'minitest/unit'
     version = MiniTest::Unit::VERSION.split(/\./).first(2).join(".")
 
