@@ -8,6 +8,18 @@ require 'rbconfig'
 
 module MiniTest
 
+  def self.const_missing name # :nodoc:
+    case name
+    when :MINI_DIR then
+      msg = "MiniTest::MINI_DIR was removed. Don't violate other's internals."
+      warn "WAR\NING: #{msg}"
+      warn "WAR\NING: Used by #{caller.first}."
+      const_set :MINI_DIR, "bad value"
+    else
+      super
+    end
+  end
+
   ##
   # Assertion base class
 
@@ -18,22 +30,6 @@ module MiniTest
 
   class Skip < Assertion; end
 
-  file = if RUBY_VERSION =~ /^1\.9/ then  # bt's expanded, but __FILE__ isn't :(
-           File.expand_path __FILE__
-         elsif  __FILE__ =~ /^[^\.]/ then # assume both relative
-           require 'pathname'
-           pwd = Pathname.new Dir.pwd
-           pn = Pathname.new File.expand_path(__FILE__)
-           relpath = pn.relative_path_from(pwd) rescue pn
-           pn = File.join ".", relpath unless pn.relative?
-           pn.to_s
-         else                             # assume both are expanded
-           __FILE__
-         end
-
-  # './lib' in project dir, or '/usr/local/blahblah' if installed
-  MINI_DIR = File.dirname(File.dirname(file)) # :nodoc:
-
   def self.filter_backtrace bt # :nodoc:
     return ["No backtrace"] unless bt
 
@@ -41,11 +37,11 @@ module MiniTest
 
     unless $DEBUG then
       bt.each do |line|
-        break if line.rindex MINI_DIR, 0
+        break if line =~ /lib\/minitest/
         new_bt << line
       end
 
-      new_bt = bt.reject { |line| line.rindex MINI_DIR, 0 } if new_bt.empty?
+      new_bt = bt.reject { |line| line =~ /lib\/minitest/ } if new_bt.empty?
       new_bt = bt.dup if new_bt.empty?
     else
       new_bt = bt.dup
@@ -59,8 +55,13 @@ module MiniTest
   # printed if the assertion fails.
 
   module Assertions
+    UNDEFINED = Object.new # :nodoc:
 
-    WINDOZE = RbConfig::CONFIG['host_os'] =~ /mswin|mingw/
+    def UNDEFINED.inspect # :nodoc:
+      "UNDEFINED" # again with the rdoc bugs... :(
+    end
+
+    WINDOZE = RbConfig::CONFIG['host_os'] =~ /mswin|mingw/ # :nodoc:
 
     ##
     # Returns the diff command to use in #diff. Tries to intelligently
@@ -223,7 +224,7 @@ module MiniTest
 
     def assert_in_delta exp, act, delta = 0.001, msg = nil
       n = (exp - act).abs
-      msg = message(msg) { "Expected #{exp} - #{act} (#{n}) to be < #{delta}" }
+      msg = message(msg) { "Expected |#{exp} - #{act}| (#{n}) to be < #{delta}"}
       assert delta >= n, msg
     end
 
@@ -247,7 +248,7 @@ module MiniTest
     end
 
     ##
-    # Fails unless +obj+ is an instace of +cls+.
+    # Fails unless +obj+ is an instance of +cls+.
 
     def assert_instance_of cls, obj, msg = nil
       msg = message(msg) {
@@ -285,9 +286,6 @@ module MiniTest
       assert obj.nil?, msg
     end
 
-    UNDEFINED = Object.new
-    def UNDEFINED.inspect; "UNDEFINED"; end
-
     ##
     # For testing with binary operators.
     #
@@ -311,8 +309,8 @@ module MiniTest
         yield
       end
 
-      x = assert_equal stdout, out, "In stdout" if stdout
       y = assert_equal stderr, err, "In stderr" if stderr
+      x = assert_equal stdout, out, "In stdout" if stdout
 
       (!stdout || x) && (!stderr || y)
     end
@@ -335,7 +333,7 @@ module MiniTest
     # Fails unless the block raises one of +exp+
 
     def assert_raises *exp
-      msg = "#{exp.pop}\n" if String === exp.last
+      msg = "#{exp.pop}.\n" if String === exp.last
 
       should_raise = false
       begin
@@ -528,7 +526,7 @@ module MiniTest
     def refute_in_delta exp, act, delta = 0.001, msg = nil
       n = (exp - act).abs
       msg = message(msg) {
-        "Expected #{exp} - #{act} (#{n}) to not be < #{delta}"
+        "Expected |#{exp} - #{act}| (#{n}) to not be < #{delta}"
       }
       refute delta > n, msg
     end
@@ -644,8 +642,8 @@ module MiniTest
     end
   end
 
-  class Unit
-    VERSION = "2.6.1" # :nodoc:
+  class Unit # :nodoc:
+    VERSION = "2.9.1" # :nodoc:
 
     attr_accessor :report, :failures, :errors, :skips # :nodoc:
     attr_accessor :test_count, :assertion_count       # :nodoc:
@@ -654,6 +652,9 @@ module MiniTest
     attr_accessor :verbose                            # :nodoc:
     attr_writer   :options                            # :nodoc:
 
+    ##
+    # Lazy accessor for options.
+
     def options
       @options ||= {}
     end
@@ -742,6 +743,9 @@ module MiniTest
                      grep(/^run_/).map { |s| s.to_s }).uniq
     end
 
+    ##
+    # Return the IO for output.
+
     def output
       self.class.output
     end
@@ -754,6 +758,9 @@ module MiniTest
       output.print(*a)
     end
 
+    ##
+    # Runner for a given +type+ (eg, test vs bench).
+
     def _run_anything type
       suites = TestCase.send "#{type}_suites"
       return if suites.empty?
@@ -791,10 +798,16 @@ module MiniTest
       status
     end
 
+    ##
+    # Runs all the +suites+ for a given +type+.
+
     def _run_suites suites, type
       suites.map { |suite| _run_suite suite, type }
     end
 
+    ##
+    # Run a single +suite+ for a given +type+.
+
     def _run_suite suite, type
       header = "#{type}_suite_header"
       puts send(header, suite) if respond_to? header
@@ -859,7 +872,7 @@ module MiniTest
       @verbose = false
     end
 
-    def process_args args = []
+    def process_args args = [] # :nodoc:
       options = {}
       orig_args = args.dup
 
@@ -942,6 +955,52 @@ module MiniTest
       io.puts format % [test_count, assertion_count, failures, errors, skips]
     end
 
+    ##
+    # Provides a simple set of guards that you can use in your tests
+    # to skip execution if it is not applicable. These methods are
+    # mixed into TestCase as both instance and class methods so you
+    # can use them inside or outside of the test methods.
+    #
+    #   def test_something_for_mri
+    #     skip "bug 1234"  if jruby?
+    #     # ...
+    #   end
+    #
+    #   if windows? then
+    #     # ... lots of test methods ...
+    #   end
+
+    module Guard
+
+      ##
+      # Is this running on jruby?
+
+      def jruby? platform = RUBY_PLATFORM
+        "java" == platform
+      end
+
+      ##
+      # Is this running on mri?
+
+      def mri? platform = RUBY_DESCRIPTION
+        /^ruby/ =~ platform
+      end
+
+      ##
+      # Is this running on rubinius?
+
+      def rubinius? platform = defined?(RUBY_ENGINE) && RUBY_ENGINE
+        "rbx" == platform
+      end
+
+      ##
+      # Is this running on windows?
+
+      def windows? platform = RUBY_PLATFORM
+        /mswin|mingw/ =~ platform
+      end
+    end
+
     ##
     # Subclass TestCase to create your own tests. Typically you'll want a
     # TestCase subclass per implementation class.
@@ -949,6 +1008,9 @@ module MiniTest
     # See MiniTest::Assertions
 
     class TestCase
+      include Guard
+      extend Guard
+
       attr_reader :__name__ # :nodoc:
 
       PASSTHROUGH_EXCEPTIONS = [NoMemoryError, SignalException,
@@ -971,7 +1033,7 @@ module MiniTest
           @passed = nil
           self.setup
           self.run_setup_hooks
-          self.__send__ self.__name__
+          self.run_test self.__name__
           result = "." unless io?
           @passed = true
         rescue *PASSTHROUGH_EXCEPTIONS
@@ -993,17 +1055,30 @@ module MiniTest
         result
       end
 
+      alias :run_test :__send__
+
       def initialize name # :nodoc:
         @__name__ = name
         @__io__ = nil
         @passed = nil
+        @@current = self
       end
 
+      def self.current # :nodoc:
+        @@current
+      end
+
+      ##
+      # Return the output IO object
+
       def io
         @__io__ = true
         MiniTest::Unit.output
       end
 
+      ##
+      # Have we hooked up the IO yet?
+
       def io?
         @__io__
       end
@@ -1088,17 +1163,17 @@ module MiniTest
       # The argument can be any object that responds to #call or a block.
       # That means that this call,
       #
-      #     MiniTest::TestCase.add_setup_hook { puts "foo" }
+      #     MiniTest::Unit::TestCase.add_setup_hook { puts "foo" }
       #
       # ... is equivalent to:
       #
       #     module MyTestSetup
-      #       def call
+      #       def self.call
       #         puts "foo"
       #       end
       #     end
       #
-      #     MiniTest::TestCase.add_setup_hook MyTestSetup
+      #     MiniTest::Unit::TestCase.add_setup_hook MyTestSetup
       #
       # The blocks passed to +add_setup_hook+ take an optional parameter that
       # will be the TestCase instance that is executing the block.
@@ -1137,17 +1212,17 @@ module MiniTest
       # The argument can be any object that responds to #call or a block.
       # That means that this call,
       #
-      #     MiniTest::TestCase.add_teardown_hook { puts "foo" }
+      #     MiniTest::Unit::TestCase.add_teardown_hook { puts "foo" }
       #
       # ... is equivalent to:
       #
       #     module MyTestTeardown
-      #       def call
+      #       def self.call
       #         puts "foo"
       #       end
       #     end
       #
-      #     MiniTest::TestCase.add_teardown_hook MyTestTeardown
+      #     MiniTest::Unit::TestCase.add_teardown_hook MyTestTeardown
       #
       # The blocks passed to +add_teardown_hook+ take an optional parameter
       # that will be the TestCase instance that is executing the block.
